@@ -59,7 +59,7 @@ SEXP Runuran_quantile (SEXP sexp_unur, SEXP sexp_U);
 /* Quantile for distribution in UNU.RAN generator object.                    */
 /*---------------------------------------------------------------------------*/
 
-SEXP Runuran_PDF (SEXP sexp_obj, SEXP sexp_x);
+SEXP Runuran_PDF (SEXP sexp_obj, SEXP sexp_x, SEXP sexp_islog);
 /*---------------------------------------------------------------------------*/
 /* Evaluate PDF or PMF for UNU.RAN distribution or generator object.         */
 /*---------------------------------------------------------------------------*/
