@@ -1,10 +1,9 @@
-/* Copyright (c) 2000-2011 Wolfgang Hoermann and Josef Leydold */
+/* Copyright (c) 2000-2012 Wolfgang Hoermann and Josef Leydold */
 /* Department of Statistics and Mathematics, WU Wien, Austria  */
 
 #include <unur_source.h>
 #include <methods/cstd.h>
 #include <methods/cstd_struct.h>
-#include <specfunct/unur_specfunct_source.h>
 #include "unur_distributions_source.h"
 inline static int beta_bc_init( struct unur_gen *gen );
 inline static int beta_bb_init( struct unur_gen *gen );
@@ -15,7 +14,6 @@ inline static int beta_b1prs_init( struct unur_gen *gen );
 #define GEN       ((struct unur_cstd_gen*)gen->datap) 
 #define DISTR     gen->distr->data.cont 
 #define uniform()  _unur_call_urng(gen->urng) 
-#define MAX_gen_params 22      
 #define p     (DISTR.params[0])   
 #define q     (DISTR.params[1])   
 #define a     (DISTR.params[2])   
@@ -37,45 +35,43 @@ _unur_stdgen_beta_init( struct unur_par *par, struct unur_gen *gen )
     }
   case 2:  
     if (gen==NULL) return UNUR_SUCCESS;  
-    if (p>1.)
-      if (q>1.) {    
-	_unur_cstd_set_sampling_routine(gen, _unur_stdgen_sample_beta_b1prs );
-	return beta_b1prs_init( gen );
-      }
-      else {         
-	_unur_cstd_set_sampling_routine(gen, _unur_stdgen_sample_beta_b01 );
-	return beta_b01_init( gen );
-      }
-    else
-      if (q>1.) {    
-	_unur_cstd_set_sampling_routine(gen, _unur_stdgen_sample_beta_b01 );
-	return beta_b01_init( gen );
-      }
-      else {         
-	_unur_cstd_set_sampling_routine(gen, _unur_stdgen_sample_beta_b00 );
-	return beta_b00_init( gen );
-      }
+    if (p > 1. && q > 1.) {
+      _unur_cstd_set_sampling_routine(gen, _unur_stdgen_sample_beta_b1prs );
+      return beta_b1prs_init( gen );
+    }
+    else if (p < 1. && q < 1.) {
+      _unur_cstd_set_sampling_routine(gen, _unur_stdgen_sample_beta_b00 );
+      return beta_b00_init( gen );
+    }
+    else if (_unur_isone(p) || _unur_isone(q)) { 
+      _unur_cstd_set_sampling_routine(gen, _unur_stdgen_sample_beta_binv );
+      return UNUR_SUCCESS;
+    }
+    else { 
+      _unur_cstd_set_sampling_routine(gen, _unur_stdgen_sample_beta_b01 );
+      return beta_b01_init( gen );
+    }
   default: 
     return UNUR_FAILURE;
   }
 } 
+#define GEN_N_PARAMS (8)
 #define am      (GEN->gen_param[0])
 #define bm      (GEN->gen_param[1])
 #define al      (GEN->gen_param[2])
 #define alnam   (GEN->gen_param[3])
 #define be      (GEN->gen_param[4])
-#define ga      (GEN->gen_param[5])
-#define si      (GEN->gen_param[6])
-#define rk1     (GEN->gen_param[7])
-#define rk2     (GEN->gen_param[8])
+#define si      (GEN->gen_param[5])
+#define rk1     (GEN->gen_param[6])
+#define rk2     (GEN->gen_param[7])
 inline static int
 beta_bc_init( struct unur_gen *gen )
 {
   CHECK_NULL(gen,UNUR_ERR_NULL);
   COOKIE_CHECK(gen,CK_CSTD_GEN,UNUR_ERR_COOKIE);
-  if (GEN->gen_param == NULL) {
-    GEN->n_gen_param = MAX_gen_params;
-    GEN->gen_param = _unur_xmalloc(GEN->n_gen_param * sizeof(double));
+  if (GEN->gen_param == NULL || GEN->n_gen_param != GEN_N_PARAMS) {
+    GEN->n_gen_param = GEN_N_PARAMS;
+    GEN->gen_param = _unur_xrealloc(GEN->gen_param, GEN->n_gen_param * sizeof(double));
   }
   am = (p > q) ? p : q;
   bm = (p < q) ? p : q;
@@ -149,14 +145,29 @@ _unur_stdgen_sample_beta_bc(  struct unur_gen *gen )
   }
   return ((DISTR.n_params==2) ? X : a + (b-a) * X);
 } 
+#undef GEN_N_PARAMS
+#undef am
+#undef bm
+#undef al
+#undef alnam
+#undef be
+#undef si
+#undef rk1
+#undef rk2
+#define GEN_N_PARAMS (5)
+#define am      (GEN->gen_param[0])
+#define bm      (GEN->gen_param[1])
+#define al      (GEN->gen_param[2])
+#define be      (GEN->gen_param[3])
+#define ga      (GEN->gen_param[4])
 inline static int
 beta_bb_init( struct unur_gen *gen )
 {
   CHECK_NULL(gen,UNUR_ERR_NULL);
   COOKIE_CHECK(gen,CK_CSTD_GEN,UNUR_ERR_COOKIE);
-  if (GEN->gen_param == NULL) {
-    GEN->n_gen_param = MAX_gen_params;
-    GEN->gen_param = _unur_xmalloc(GEN->n_gen_param * sizeof(double));
+  if (GEN->gen_param == NULL || GEN->n_gen_param != GEN_N_PARAMS) {
+    GEN->n_gen_param = GEN_N_PARAMS;
+    GEN->gen_param = _unur_xrealloc(GEN->gen_param, GEN->n_gen_param * sizeof(double));
   }
   am = (p < q) ? p : q;
   bm = (p > q) ? p : q;
@@ -191,15 +202,13 @@ _unur_stdgen_sample_beta_bb(  struct unur_gen *gen )
   }
   return ((DISTR.n_params==2) ? X : a + (b-a) * X);
 } 
+#undef GEN_N_PARAMS
 #undef am
 #undef bm
 #undef al
-#undef alnam
 #undef be
 #undef ga
-#undef si
-#undef rk1
-#undef rk2
+#define GEN_N_PARAMS (8)
 #define p_      (GEN->gen_param[0])
 #define q_      (GEN->gen_param[1])
 #define c       (GEN->gen_param[2])
@@ -213,9 +222,9 @@ beta_b00_init( struct unur_gen *gen )
 {
   CHECK_NULL(gen,UNUR_ERR_NULL);
   COOKIE_CHECK(gen,CK_CSTD_GEN,UNUR_ERR_COOKIE);
-  if (GEN->gen_param == NULL) {
-    GEN->n_gen_param = MAX_gen_params;
-    GEN->gen_param = _unur_xmalloc(GEN->n_gen_param * sizeof(double));
+  if (GEN->gen_param == NULL || GEN->n_gen_param != GEN_N_PARAMS) {
+    GEN->n_gen_param = GEN_N_PARAMS;
+    GEN->gen_param = _unur_xrealloc(GEN->gen_param, GEN->n_gen_param * sizeof(double));
   }
   p_ = p - 1.;
   q_ = q - 1.;
@@ -260,6 +269,7 @@ _unur_stdgen_sample_beta_b00(  struct unur_gen *gen )
   }
   return ((DISTR.n_params==2) ? X : a + (b-a) * X);
 } 
+#undef GEN_N_PARAMS
 #undef p_
 #undef q_
 #undef c
@@ -268,6 +278,7 @@ _unur_stdgen_sample_beta_b00(  struct unur_gen *gen )
 #undef fq
 #undef p1
 #undef p2
+#define GEN_N_PARAMS (11)
 #define pint    (GEN->gen_param[0])
 #define qint    (GEN->gen_param[1])
 #define p_      (GEN->gen_param[2])
@@ -284,9 +295,9 @@ beta_b01_init( struct unur_gen *gen )
 {
   CHECK_NULL(gen,UNUR_ERR_NULL);
   COOKIE_CHECK(gen,CK_CSTD_GEN,UNUR_ERR_COOKIE);
-  if (GEN->gen_param == NULL) {
-    GEN->n_gen_param = MAX_gen_params;
-    GEN->gen_param = _unur_xmalloc(GEN->n_gen_param * sizeof(double));
+  if (GEN->gen_param == NULL || GEN->n_gen_param != GEN_N_PARAMS) {
+    GEN->n_gen_param = GEN_N_PARAMS;
+    GEN->gen_param = _unur_xrealloc(GEN->gen_param, GEN->n_gen_param * sizeof(double));
   }
   if (p>q) {
     pint = q;
@@ -349,6 +360,7 @@ _unur_stdgen_sample_beta_b01(  struct unur_gen *gen )
     X = 1. - X;
   return ((DISTR.n_params==2) ? X : a + (b-a) * X);
 } 
+#undef GEN_N_PARAMS
 #undef pint
 #undef qint
 #undef p_
@@ -360,6 +372,7 @@ _unur_stdgen_sample_beta_b01(  struct unur_gen *gen )
 #undef mu
 #undef p1
 #undef p2
+#define GEN_N_PARAMS (22)
 #define p_      (GEN->gen_param[0])
 #define q_      (GEN->gen_param[1])
 #define s       (GEN->gen_param[2])
@@ -387,9 +400,9 @@ beta_b1prs_init( struct unur_gen *gen )
 {
   CHECK_NULL(gen,UNUR_ERR_NULL);
   COOKIE_CHECK(gen,CK_CSTD_GEN,UNUR_ERR_COOKIE);
-  if (GEN->gen_param == NULL) {
-    GEN->n_gen_param = MAX_gen_params;
-    GEN->gen_param = _unur_xmalloc(GEN->n_gen_param * sizeof(double));
+  if (GEN->gen_param == NULL || GEN->n_gen_param != GEN_N_PARAMS) {
+    GEN->n_gen_param = GEN_N_PARAMS;
+    GEN->gen_param = _unur_xrealloc(GEN->gen_param, GEN->n_gen_param * sizeof(double));
   }
   p_ = p - 1.0;
   q_ = q - 1.0;
@@ -535,6 +548,7 @@ _unur_stdgen_sample_beta_b1prs(  struct unur_gen *gen )
   }
   return ((DISTR.n_params==2) ? X : a + (b-a) * X);
 } 
+#undef GEN_N_PARAMS
 #undef p_
 #undef q_
 #undef s 
@@ -557,3 +571,20 @@ _unur_stdgen_sample_beta_b1prs(  struct unur_gen *gen )
 #undef p2
 #undef p3
 #undef p4
+double 
+_unur_stdgen_sample_beta_binv(  struct unur_gen *gen )
+{
+  double X;
+  CHECK_NULL(gen,INFINITY);
+  COOKIE_CHECK(gen,CK_CSTD_GEN,INFINITY);
+  if (_unur_isone(p) && _unur_isone(q)) {
+    X = uniform();
+  }
+  else if (_unur_isone(p)) {
+    X = 1. - pow(1.-uniform(), 1/q);
+  }
+  else { 
+    X = pow(uniform(), 1/p);
+  }
+  return ((DISTR.n_params==2) ? X : a + (b-a) * X);
+} 
