@@ -1,18 +1,16 @@
-/* Copyright (c) 2000-2011 Wolfgang Hoermann and Josef Leydold */
+/* Copyright (c) 2000-2012 Wolfgang Hoermann and Josef Leydold */
 /* Department of Statistics and Mathematics, WU Wien, Austria  */
 
 #include <limits.h>
 #include <unur_source.h>
 #include <methods/cstd.h>   
 #include <methods/dstd_struct.h>
-#include <specfunct/unur_specfunct_source.h>
 #include "unur_distributions_source.h"
 inline static int zipf_zet_init( struct unur_gen *gen );
 #define PAR       ((struct unur_dstd_par*)par->datap) 
 #define GEN       ((struct unur_dstd_gen*)gen->datap) 
 #define DISTR     gen->distr->data.discr 
 #define uniform()  _unur_call_urng(gen->urng) 
-#define MAX_gen_params  2      
 #define rho  (DISTR.params[0])    
 #define tau  (DISTR.params[1])
 int 
@@ -27,6 +25,7 @@ _unur_stdgen_zipf_init( struct unur_par *par, struct unur_gen *gen )
     return UNUR_FAILURE;
   }
 } 
+#define GEN_N_PARAMS  (2)
 #define c   (GEN->gen_param[0])
 #define d   (GEN->gen_param[1])
 inline static int
@@ -34,9 +33,9 @@ zipf_zet_init( struct unur_gen *gen )
 {
   CHECK_NULL(gen,UNUR_ERR_NULL);
   COOKIE_CHECK(gen,CK_DSTD_GEN,UNUR_ERR_COOKIE);
-  if (GEN->gen_param == NULL) {
-    GEN->n_gen_param = MAX_gen_params;
-    GEN->gen_param = _unur_xmalloc(GEN->n_gen_param * sizeof(double));
+  if (GEN->gen_param == NULL || GEN->n_gen_param != GEN_N_PARAMS) {
+    GEN->n_gen_param = GEN_N_PARAMS;
+    GEN->gen_param = _unur_xrealloc(GEN->gen_param, GEN->n_gen_param * sizeof(double));
   }
   if (rho<tau) {
     c = tau - 0.5;
@@ -66,5 +65,6 @@ _unur_stdgen_sample_zipf_zet( struct unur_gen *gen )
   } while ( E < (1.+rho) * log( (K+tau)/(X+c)) - d );
   return K;
 } 
+#undef GEN_N_PARAMS
 #undef c
 #undef d
