@@ -1,4 +1,4 @@
-/* Copyright (c) 2000-2011 Wolfgang Hoermann and Josef Leydold */
+/* Copyright (c) 2000-2012 Wolfgang Hoermann and Josef Leydold */
 /* Department of Statistics and Mathematics, WU Wien, Austria  */
 
 #undef __BEGIN_DECLS
@@ -596,6 +596,59 @@ void *unur_dext_get_params( UNUR_GEN *generator, size_t size );
 double *unur_dext_get_distrparams( UNUR_GEN *generator );
 int unur_dext_get_ndistrparams( UNUR_GEN *generator );
 UNUR_PAR *unur_unif_new( const UNUR_DISTR *dummy );
+#ifndef UNUR_METHODS_H_SEEN
+#define UNUR_METHODS_H_SEEN
+#define UNUR_MASK_TYPE     0xff000000u   
+#define UNUR_METH_DISCR    0x01000000u
+#define UNUR_METH_DARI     0x01000001u
+#define UNUR_METH_DAU      0x01000002u
+#define UNUR_METH_DGT      0x01000003u
+#define UNUR_METH_DSROU    0x01000004u
+#define UNUR_METH_DSS      0x01000005u
+#define UNUR_METH_CONT     0x02000000u
+#define UNUR_METH_AROU     0x02000100u
+#define UNUR_METH_ARS      0x02000d00u
+#define UNUR_METH_HINV     0x02000200u
+#define UNUR_METH_HRB      0x02000300u
+#define UNUR_METH_HRD      0x02000400u
+#define UNUR_METH_HRI      0x02000500u
+#define UNUR_METH_ITDR     0x02000800u
+#define UNUR_METH_NINV     0x02000600u
+#define UNUR_METH_NROU     0x02000700u
+#define UNUR_METH_PINV     0x02001000u
+#define UNUR_METH_SROU     0x02000900u
+#define UNUR_METH_SSR      0x02000a00u
+#define UNUR_METH_TABL     0x02000b00u
+#define UNUR_METH_TDR      0x02000c00u
+#define UNUR_METH_UNIF     0x02000e00u
+#define UNUR_METH_UTDR     0x02000f00u
+#define UNUR_METH_CEMP     0x04000000u
+#define UNUR_METH_EMPK     0x04001100u
+#define UNUR_METH_EMPL     0x04001200u
+#define UNUR_METH_HIST     0x04001300u
+#define UNUR_METH_VEC      0x08000000u
+#define UNUR_METH_MVTDR    0x08010000u
+#define UNUR_METH_VMT      0x08020000u
+#define UNUR_METH_VNROU    0x08030000u
+#define UNUR_METH_VAROU    0x08040000u
+#define UNUR_METH_NORTA    0x08050000u
+#define UNUR_METH_GIBBS    0x08060000u
+#define UNUR_METH_HITRO    0x08070000u
+#define UNUR_METH_BALL     0x08080000u
+#define UNUR_METH_WALK     0x08090000u
+#define UNUR_METH_CVEMP    0x10000000u
+#define UNUR_METH_VEMPK    0x10010000u
+#define UNUR_METH_MAT      0x20000000u
+#define UNUR_METH_MCORR    0x20010000u
+#define UNUR_METH_CSTD     0x0200f100u   
+#define UNUR_METH_DSTD     0x0100f200u   
+#define UNUR_METH_MVSTD    0x0800f300u   
+#define UNUR_METH_MIXT     0x0200e100u   
+#define UNUR_METH_CEXT     0x0200f400u   
+#define UNUR_METH_DEXT     0x0100f500u   
+#define UNUR_METH_AUTO     0x00a00000u   
+#define UNUR_METH_UNKNOWN  0xff000000u
+#endif  
 #ifndef UNUR_DEPRECATED_DISTR_H_SEEN
 #define UNUR_DEPRECATED_DISTR_H_SEEN
 int unur_distr_cvec_set_stdmarginals( UNUR_DISTR *distribution, UNUR_DISTR *marginal );
@@ -687,6 +740,8 @@ void  unur_free( UNUR_GEN *generator );
 const char *unur_gen_info( UNUR_GEN *generator, int help );
 int unur_get_dimension( const UNUR_GEN *generator );
 const char *unur_get_genid( const UNUR_GEN *generator );
+unsigned int unur_get_method( const UNUR_GEN *generator );
+int unur_gen_is_inversion ( const UNUR_GEN *gen );
 UNUR_DISTR *unur_get_distr( const UNUR_GEN *generator );
 int unur_set_use_distr_privatecopy( UNUR_PAR *parameters, int use_privatecopy );
 UNUR_GEN *unur_gen_clone( const UNUR_GEN *gen );
@@ -720,6 +775,7 @@ enum {
   UNUR_DISTR_LOGISTIC         = 0x00000d01u,  
   UNUR_DISTR_LOGNORMAL        = 0x00000e01u,  
   UNUR_DISTR_LOMAX            = 0x00000f01u,  
+  UNUR_DISTR_MEIXNER          = 0x00002601u,  
   UNUR_DISTR_NORMAL           = 0x00001001u,  
    UNUR_DISTR_GAUSSIAN        = 0x00001001u,  
   UNUR_DISTR_PARETO           = 0x00001101u,  
@@ -730,6 +786,7 @@ enum {
   UNUR_DISTR_TRIANGULAR       = 0x00001601u,  
   UNUR_DISTR_UNIFORM          = 0x00002001u,  
    UNUR_DISTR_BOXCAR          = 0x00002001u,  
+  UNUR_DISTR_VG               = 0x00002501u,  
   UNUR_DISTR_WEIBULL          = 0x00001801u,  
   UNUR_DISTR_BURR_I           = 0x0000b001u,  
   UNUR_DISTR_BURR_II          = 0x0000b101u,  
@@ -777,6 +834,7 @@ UNUR_DISTR *unur_distr_laplace(const double *params, int n_params);
 UNUR_DISTR *unur_distr_logistic(const double *params, int n_params);
 UNUR_DISTR *unur_distr_lognormal(const double *params, int n_params);
 UNUR_DISTR *unur_distr_lomax(const double *params, int n_params);
+UNUR_DISTR *unur_distr_meixner(const double *params, int n_params);
 UNUR_DISTR *unur_distr_normal( const double *params, int n_params );
 UNUR_DISTR *unur_distr_pareto( const double *params, int n_params );
 UNUR_DISTR *unur_distr_powerexponential(const double *params, int n_params);
@@ -785,6 +843,7 @@ UNUR_DISTR *unur_distr_slash(const double *params, int n_params);
 UNUR_DISTR *unur_distr_student(const double *params, int n_params);
 UNUR_DISTR *unur_distr_triangular(const double *params, int n_params);
 UNUR_DISTR *unur_distr_uniform(const double *params, int n_params);
+UNUR_DISTR *unur_distr_vg(const double *params, int n_params);
 UNUR_DISTR *unur_distr_weibull(const double *params, int n_params);
 UNUR_DISTR *unur_distr_multinormal(int dim, const double *mean, const double *covar);
 UNUR_DISTR *unur_distr_multicauchy(int dim, const double *mean, const double *covar);
